#include <string.h>
#include <windows.h>
#include <winsock.h>
#include "mailbox.h"
#include "acidmail.h"
#include "version.h"
#include "resource.h"
#include "..\litestep\lsapi\lsapi.h"

BOOL CALLBACK GetPassProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM /* lParam */);
extern HINSTANCE hwndInstance;
char temp_name[64] = {0}, temp_pass[64] = {0};

Mailbox::Mailbox(char* name, char* type, char* server, char* user, char* pass):
	mail()
{
	// Copy args
	strcpy(sname, name);
	if (*type) // One line syntax
	{
		strcpy(sType, type);
		strcpy(sUser, user);
		char* p = strtok(server, ":");
		strcpy(sServer, p);
		p = strtok(NULL, "");
		if (p) iPort = atoi(p);
		else if (!strcmp("imap", sType)) iPort = 143;
		else iPort = 110;
		strcpy(sPass, pass);
		strcpy(sFolder, "inbox"); // Can't be set with this syntax
	}
	else // Multiline syntax
	{
		strcpy(sSetting, name);
		strcat(sSetting, "server");
		if(!GetRCString(sSetting, sServer, NULL, MAX_LINE_LENGTH))
		{
			ErrorHandler(3, NULL);
			mail.bError = true;
		}
		else
		{
			strcpy(sSetting, name);
			strcat(sSetting, "type");
			GetRCString(sSetting, sType, "pop3", MAX_LINE_LENGTH);
			strcpy(sSetting, name);
			strcat(sSetting, "user");
			GetRCString(sSetting, sUser, "anonymous", MAX_LINE_LENGTH);
			strcpy(sSetting, name);
			strcat(sSetting, "port");
			if (!strcmp("imap", sType))
				iPort = GetRCInt(sSetting, 143);
			else iPort = GetRCInt(sSetting, 110);
			strcpy(sSetting, name);
			strcat(sSetting, "password");
			GetRCString(sSetting, sPass, "", MAX_LINE_LENGTH);
			strcpy(sSetting, name);
			strcat(sSetting, "folder");
			GetRCString(sSetting, sFolder, "inbox", MAX_LINE_LENGTH);
		}
	}
	if (!mail.bError)
	{
		if (!(*sPass)) // If no password was found in config, get it from database/dialog
		{
			char temp[256];
			sprintf(temp, "%s/%s", sServer, sUser);
			GetProfileString("AcidMail", temp, "", temp, 256);
			if (!strlen(temp))
			{
				strcpy(temp_name, sname);
				DialogBox(hwndInstance, MAKEINTRESOURCE(IDD_GETPASS), NULL, GetPassProc);			
				if (strlen(temp_pass)) 
				{
					strcpy(sPass, temp_pass);
					sprintf(temp, "%s/%s", sServer, sUser);
					WriteProfileString("AcidMail", temp, Encrypt(sPass));
				}
				memset(&(temp_pass), 0, sizeof(temp_pass));
			}
			else
				strcpy(sPass, Encrypt(temp));
		}

		// Set evars
		strcpy(sEvar, "AcidMail");
		strcat(sEvar, name);
		LSSetVariable(sEvar, "0");
	}
}

Mailbox::~Mailbox()
{

}

Mail Mailbox::CheckMail()
{
	SOCKET s;
	try
	{
		WSADATA wsa;
		if (WSAStartup(MAKEWORD(1,1),&wsa))
			throw 1;	
		HOSTENT*  dns;
		if (isalpha(sServer[0]))
			dns = gethostbyname(sServer);
		else
		{
			unsigned int addr = inet_addr(sServer); 
			dns = gethostbyaddr((char *)&addr,4,AF_INET);
		}
		if(!dns) throw 2; 
		char* &ip=dns->h_addr;
		sockaddr_in  a={AF_INET,htons(iPort), ip[0],ip[1],ip[2],ip[3]};
		s=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
		if(connect(s,(sockaddr*)&a,sizeof(a)) <0 ) throw 6;
		if (!strcmp("pop3", sType))
			mail.iNumMail = Pop3Check(s);
		else if(!strcmp("imap", sType))
			mail.iNumMail = ImapCheck(s);
		else
			throw 9;
		closesocket(s);
		WSACleanup();
		char snum[8];
		sprintf(snum, "%d", mail.iNumMail);
		LSSetVariable(sEvar, snum);
		mail.bError = false;
	}
	catch (int e)
	{
		ErrorHandler(e, s);
		mail.bError = true;
	}
	return mail;
}

int Mailbox::Pop3Check(const SOCKET& s)
{
	char data[128];
	char* buf = data;
	int size=128;

	sprintf(buf, "");
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) < 0) throw 7;
	if(recv(s, buf, size, 0) <= 0 || *buf != '+' ) throw 7;

	sprintf(buf, "user %s\r\n" ,sUser);
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;
	if(*buf != '+') throw 8;

	sprintf(buf, "pass %s\r\n" ,sPass);
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;
	if(*buf != '+') throw 8;

	sprintf(buf, "stat\r\n");
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0 || *buf != '+' ) throw 7;

	send(s, "quit\r\n", 6, 0);
	char test[6];
	strcpy(test, strtok(buf, " "));
	unsigned int ans;
	if(test != NULL)
		ans = atoi(strtok(NULL, " "));
	else
		throw 255;
	recv(s, buf, size, 0);
	return ans;
}

int Mailbox::ImapCheck(const SOCKET& s)
{
	char data[1024];
	char* buf = data;
	int size=1024;
	unsigned int ans;

	sprintf(buf, "");
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) < 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;

	sprintf(buf, "a01 login %s %s\r\n", sUser, sPass);
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;

	sprintf(buf, "a02 examine %s\r\n", sFolder);
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;

	char testbuf[256];
	if(strstr(buf, "RECENT"))
	{
		strcpy(testbuf, strtok(buf, "*"));
		strcpy(testbuf, strtok(NULL, "*"));
		strcpy(testbuf, strtok(NULL, "*"));
		ans = atoi(strtok(NULL, " "));
	}
	else
		throw 255;
	send(s, "a03 logout\r\n", 12, 0);
	recv(s, buf, size, 0);
	return ans;
}

void Mailbox::ClearMail()
{
	mail.iNumMail = 0;
	mail.bError = false;
	LSSetVariable(sEvar, "0");
}

void Mailbox::ErrorHandler(const int& errnum, const SOCKET& s)
{
	if (errnum > 5)
	{
		closesocket(s);
		WSACleanup();
	}	
	switch(errnum)
	{
		case 1:
			LSLogPrintf(LOG_ERROR, szAppName, "Could not initialize winsock.");
			break;
		case 2:
			LSLogPrintf(LOG_WARNING, szAppName, "Could not find host: %s", sServer);
			break;
		case 3:
			LSLogPrintf(LOG_ERROR, szAppName, "No host entered for mailbox: %s", sname);
			break;
		case 6:
			LSLogPrintf(LOG_WARNING, szAppName, "Could not connect to host: %s", sServer);
			break;
		case 7:
			LSLogPrintf(LOG_WARNING, szAppName, "Connection error with mailbox: %s", sname);
			break;
		case 8:
			LSLogPrintf(LOG_WARNING, szAppName, "Invalid username/password");
			break;
		case 9:
			LSLogPrintf(LOG_WARNING, szAppName, "Invalid mailbox type");
			break;
		default:
			LSLogPrintf(LOG_WARNING, szAppName, "An error accured with mailbox: %s", sname);
	}
}

char* Mailbox::Encrypt(char* pass)
{
	char* ret=_strdup(pass);
	char* p=ret;
	for (; *pass; *pass++, *ret++)
	{
		*ret=*pass ^ 0x5405;
	}
	return p;
}

BOOL CALLBACK GetPassProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM /* lParam */)
{
	switch (msg)
	{
		case WM_INITDIALOG: 
		{
			char temp[256] = "";
			sprintf(temp, "%s Password", temp_name);
			SetWindowText(hwnd, temp);
			return TRUE;
		}

		case WM_COMMAND:
		{
			if (HIWORD(wParam) == BN_CLICKED)
			{
				switch (LOWORD(wParam))
				{
					case IDOK:
					{
						GetDlgItemText(hwnd, IDC_PASSWORD, temp_pass, 64);
					}
					case IDCANCEL:
					{
						EndDialog(hwnd, 0);
					}
					break;
				}
			}
		}
		break;
	}

	return false;
}

Mail::Mail()
{
	iNumMail = 0;
	bError = false;
}